@@ -2,6 +2,7 @@ package FTN::Packet;
 
 use strict;
 use warnings;
+use Carp qw( croak );
 
 =head1 NAME
 
@@ -9,11 +10,11 @@ FTN::Packet - Reading or writing Fidonet Technology Networks (FTN) packets.
 
 =head1 VERSION
 
-VERSION 0.13
+VERSION 0.20
 
 =cut
 
-our $VERSION = '0.13';
+our $VERSION = '0.20';
 
 =head1 DESCRIPTION
 
@@ -35,8 +36,6 @@ our @ISA = qw(Exporter AutoLoader);
 # Items to export into callers namespace by default. Note: do not export
 # names by default without a very good reason. Use EXPORT_OK instead.
 # Do not simply export all your public functions/methods/constants.
-our @EXPORT = qw(
-);
 our @EXPORT_OK = qw( &read_ftn_packet &write_ftn_packet
 );
 
@@ -44,22 +43,30 @@ our @EXPORT_OK = qw( &read_ftn_packet &write_ftn_packet
 
 =head2 read_ftn_packet
 
-Syntax:  $messages = read_ftn_packet(*PKT);
+Syntax:  $messages = read_ftn_packet($pkt_file);
 
-Read a Fidonet/FTN packet.  Returns the messages in the packet as a reference
-to an array of hash references, which can be read as follows:
+Read the messages in a Fidonet/FTN packet. It is passed the name and path of a
+Fidonet/FTN packet file. Returns the messages in the packet as a reference to an
+array of hashes, which can be read as follows:
 
-    $message_ref = pop(@{$messages});
-    $msg_area = ${$message_ref}->('area');
-    $msg_date = ${$message_ref}->('ftscdate');
-    $msg_tonode = ${$message_ref}->('tonode');
-    $msg_from = ${$message_ref}->('from');
-    $msg_body = ${$message_ref}->('to');
-    $msg_subj = ${$message_ref}->('subj');
-    $msg_msgid = ${$message_ref}->('msgid');
-    $msg_replyid = ${$message_ref}->('replyid');
-    $msg_body = ${$message_ref}->('body');
-    $msg_ctrl = ${$message_ref}->('ctrlinfo');
+    for $i ( 0 .. $#{$messages} ) {
+
+        print "On message $i";
+
+        $msg_area = ${$messages}[i]{area};
+        $msg_date = ${$messages}[i]{ftscdate};
+        $msg_tonode = ${$messages}[i]{tonode};
+        $msg_from = ${$messages}[i]{from};
+        $msg_to = ${$messages}[i]{to};
+        $msg_subj = ${$messages}[i]{subj};
+        $msg_msgid = ${$messages}[i]{msgid};
+        $msg_replyid = ${$messages}[i]{replyid};
+        $msg_body = ${$messages}[i]{body};
+        $msg_ctrl = ${$messages}[i]{ctrlinfo};
+
+        # Processing of the contents of the message.
+
+    }
 
 =cut
 
@@ -68,15 +75,17 @@ to an array of hash references, which can be read as follows:
 ###############################################
 sub read_ftn_packet {
 
-    my ($PKT) = @_;
-    # "$PKT" is a file pointer to the packet file being read
-    # Returns an array of hash references
+    my ($packet_file) = @_;
 
     my ($packet_version,$origin_node,$destination_node,$origin_net,$destination_net,$attribute,$cost,$buffer);
-    my ($separator, $s, $date_time, $to, $from, $subject, $area, @lines, @kludges,
+    my ($separator, $s, $date_time, $to, $from, $subject, $area, @lines, @kludges, $PKT,
         $from_node, $to_node, @messages, $message_body, $message_id, $reply_id, $origin,
         $mailer, $seen_by, $i, $k);
 
+    # "$PKT" is a file pointer to the packet file being read
+    open( $PKT, q{<}, $packet_file ) or croak("Problem opening packet file: $packet_file");
+    binmode($PKT);
+
     # Ignore packet header
     read($PKT,$buffer,58);
 
@@ -108,10 +117,10 @@ sub read_ftn_packet {
         $subject = <$PKT>;
 
         $to   =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
-        $to   =~ tr/\0-\037/\040-\100/;     # mask control characters
+        $to   =~ tr/\0-\037/\040-\077/;     # mask control characters
         $from =~ tr/\200-\377/\0-\177/;     # mask hi-bit characters
-        $from =~ tr/\0-\037/\040-\100/;     # mask control characters
-        $subject =~ tr/\0-\037/\040-\100/;     # mask control characters
+        $from =~ tr/\0-\037/\040-\077/;     # mask control characters
+        $subject =~ tr/\0-\037/\040-\077/;     # mask control characters
 
         $s = <$PKT>;
         local $/ = $separator;
@@ -172,7 +181,7 @@ sub read_ftn_packet {
         $message_body = "";
 
         foreach my $s (@lines) {
-            $s =~ tr/\0-\037/\040-\100/;
+            $s =~ tr/\0-\037/\040-\077/;     # mask control characters
             $s =~ s/\s+$//;
             $s=~tr/^\*/ /;
             $message_body .= "$s\n";
@@ -193,7 +202,7 @@ sub read_ftn_packet {
                 $message_id = substr($c, 7);
             }
 
-            $control_info .= "$s\n";
+            $control_info .= "$c\n";
         }
 
         if ( ! $message_id) {
@@ -223,8 +232,8 @@ sub read_ftn_packet {
             to => $to,
             subj => $subject,
 
-            msgid => $message_id,    
-            replyid => $reply_id,  
+            msgid => $message_id,
+            replyid => $reply_id,
 
             body => $message_body,
 
@@ -324,7 +333,7 @@ sub write_ftn_packet {
 
     $packet_file = sprintf("%s/%02d%02d%02d%02d.pkt",$OutDir,$day,$hour,$minutes,$seconds);
 
-    open( $PKT, q{>}, "$packet_file" ) || die;
+    open( $PKT, q{>}, "$packet_file" ) or croak('Cannot open FTN packet file for writing.');
 
     binmode($PKT);
 
@@ -409,9 +418,9 @@ Robert James Clay, jame@rocasa.us
 =head1 BUGS
 
 Please report any bugs or feature requests via the web interface at
-L<https://github.com/jame/ftn-packet/issues>. I will be notified,
-and then you'll automatically be notified of progress on your bug
-as I make changes.
+L<<http://sourceforge.net/p/ftnpl/ftn-packet/tickets/>. I will be
+notified, and then you'll automatically be notified of progress on
+your bug as I make changes.
 
 Note that you can also report any bugs or feature requests to
 C<bug-ftn-packet at rt.cpan.org>, or through the web interface at
@@ -430,7 +439,7 @@ You can also look for information at:
 
 =item * FTN::Packet issue tracker
 
-L<https://github.com/jame/ftn-packet/issues>
+L<http://sourceforge.net/p/ftnpl/ftn-packet/tickets/>
 
 =item * RT: CPAN's request tracker
 
@@ -455,11 +464,11 @@ scripts, also at the SourceForge project.
 
 =head1 SEE ALSO
 
- L<perl(1)>
+ L<FTN::Packet::Examples>, L<FTN::Packet::ToDo>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2001-2011 Robert James Clay, all rights reserved.
+Copyright 2001-2012 Robert James Clay, all rights reserved.
 Copyright 2001-2003 Russ Johnson, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
